"""GRPC Utils."""

from __future__ import annotations

import re
import time
from typing import TYPE_CHECKING, List, Optional, Tuple, Union

from .proto3 import Import, Package, Service
from .proto3 import proto as parser

if TYPE_CHECKING:
    from pathlib import Path

RE_PROTO_COMMENT = re.compile(r"//.*?\n+", re.S)
RE_IMPORT_PB2 = re.compile(r"^import (\S+)_pb2", re.MULTILINE)


def _parse_proto(proto: Path) -> Tuple[Optional[str], List[str], List[str]]:
    package = None
    services = []
    imports = []
    for st in parser.parse(RE_PROTO_COMMENT.sub("", proto.read_text())).statements:
        if isinstance(st, Service):
            services.append(st.name)

        elif isinstance(st, Import):
            imports.append(st.identifier)

        elif isinstance(st, Package):
            package = st.identifier[0]

    return package, services, imports


def _is_newer(target: Path, ts: float) -> bool:
    return target.is_file() and target.stat().st_ctime > ts


def _fix_imports(*targets: Path):
    for target in targets:
        text = target.read_text()
        text = RE_IMPORT_PB2.sub(r"from . import \1_pb2", text)
        target.write_text(text)


def _generate_file(target: Path, *lines: Union[str, bool]):
    timestamp = time.time()
    target.write_text(
        f"# {timestamp:.0f}: Generated by the Muffin GRPC Plugin\n"
        + "\n".join([str(line) for line in lines if line]),
    )
